@@ -329,7 +329,7 @@ class DepsolveTests(DepsolveTests):
         self.xsack.addPackage(xpo64)
 
         self.assertEquals('ok', *self.resolveCode())
-        if os.uname()[-1] == 'x86_64':
+        if self.canonArch == 'x86_64':
             self.assertResult((po, xpo64))
         else:
             self.assertResult((po, xpo))
@@ -648,7 +648,7 @@ class DepsolveTests(DepsolveTests):
 
     def testCompareProvidersSameLen2_noarch_to_64_1(self):
         # Make sure they are still ok, the other way around
-        myarch = arch.getBaseArch(arch.getCanonArch())
+        myarch = self.canonArch
 
         if myarch not in ('i386', 'x86_64'):
             return
@@ -675,7 +675,7 @@ class DepsolveTests(DepsolveTests):
 
     def testCompareProvidersSameLen2_noarch_to_64_2(self):
         # Make sure they are still ok, the other way around
-        myarch = arch.getBaseArch(arch.getCanonArch())
+        myarch = self.canonArch
 
         if myarch not in ('i386', 'x86_64'):
             return
@@ -865,10 +865,28 @@ class DepsolveTests(DepsolveTests):
         self.xsack.addPackage(po3)
 
         self.assertEquals('ok', *self.resolveCode())
-        # FIXME: This is wrong, it should be one of:
-        # self.assertResult((xpo, po3))
-        # self.assertResult((xpo, po2))
-        self.assertResult((xpo, po2, po3))
+        self.assertResult((xpo, po2))
+
+    def testMultiPkgVersions5(self):
+        ipo1 = FakePackage('abcd', arch='i386')
+        ipo1.addRequires('Foo', 'EQ', ('0', '1', '1'))
+        self.rpmdb.addPackage(ipo1)
+        ipo2 = FakePackage('Foo', arch='i386')
+        self.rpmdb.addPackage(ipo2)
+
+        xpo = FakePackage('abcd', version='2', arch='i386')
+        xpo.addRequires('Foo', 'GE', ('0', '2', '1'))
+        self.tsInfo.addUpdate(xpo, oldpo=ipo1)
+
+        po1 = FakePackage('Foo', arch='i386')
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('Foo', version='2', arch='i686')
+        po3 = FakePackage('Foo', version='2', arch='i386')
+        self.xsack.addPackage(po3)
+        self.xsack.addPackage(po2)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((xpo, po2))
 
     # Test from "Real Life" because we just can't think like they do
     def testRL_unison1(self):
@@ -904,3 +922,258 @@ class DepsolveTests(DepsolveTests):
 
         self.assertEquals('ok', *self.resolveCode())
         self.assertResult((xpo, po2))
+
+    def test_min_inst_and_dep(self):
+        ipo1 = FakePackage('bar', version='2')
+        self.tsInfo.addInstall(ipo1)
+
+        ipo2 = FakePackage('foo')
+        ipo2.addRequires('bar', 'GE', (None, '3', '0'))
+        self.tsInfo.addInstall(ipo2)
+
+        po1 = FakePackage('foo')
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('bar', version='2')
+        self.xsack.addPackage(po2)
+        po3 = FakePackage('bar', version='3')
+        self.xsack.addPackage(po3)
+        po4 = FakePackage('bar', version='4')
+        self.xsack.addPackage(po4)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo2, po4))
+
+    def test_min_up_and_dep1(self):
+        rpo1 = FakePackage('bar', version='1')
+        self.rpmdb.addPackage(rpo1)
+
+        ipo1 = FakePackage('bar', version='2')
+        self.tsInfo.addUpdate(ipo1, oldpo=rpo1)
+
+        ipo2 = FakePackage('foo')
+        ipo2.addRequires('bar', 'GE', (None, '3', '0'))
+        self.tsInfo.addInstall(ipo2)
+
+        po1 = FakePackage('foo')
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('bar', version='2')
+        self.xsack.addPackage(po2)
+        po3 = FakePackage('bar', version='3')
+        self.xsack.addPackage(po3)
+        po4 = FakePackage('bar', version='4')
+        self.xsack.addPackage(po4)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo2, po4))
+
+    def test_min_up_and_dep2(self):
+        rpo1 = FakePackage('bar', version='1')
+        self.rpmdb.addPackage(rpo1)
+
+        ipo1 = FakePackage('bar', version='2')
+
+        ipo2 = FakePackage('foo')
+        ipo2.addRequires('bar', 'GE', (None, '3', '0'))
+        self.tsInfo.addInstall(ipo2)
+        self.tsInfo.addUpdate(ipo1, oldpo=rpo1)
+
+        po1 = FakePackage('foo')
+        po2 = FakePackage('bar', version='2')
+        po3 = FakePackage('bar', version='3')
+        po4 = FakePackage('bar', version='4')
+        self.xsack.addPackage(po4)
+        self.xsack.addPackage(po3)
+        self.xsack.addPackage(po2)
+        self.xsack.addPackage(po1)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo2, po4))
+
+    def test_min_up_and_dep3(self):
+        rpo1 = FakePackage('bar', version='1')
+        self.rpmdb.addPackage(rpo1)
+        rpo2 = FakePackage('bar-blah', version='1')
+        rpo2.addRequires('bar', 'EQ', ('0', '1', '1'))
+        self.rpmdb.addPackage(rpo2)
+
+        ipo1 = FakePackage('bar', version='2')
+        self.tsInfo.addUpdate(ipo1, oldpo=rpo1)
+        ipo2 = FakePackage('bar-blah', version='2')
+        ipo2.addRequires('bar', 'EQ', ('0', '2', '1'))
+        self.tsInfo.addUpdate(ipo2, oldpo=rpo2)
+
+        ipo3 = FakePackage('foo')
+        ipo3.addRequires('bar', 'GE', (None, '3', '0'))
+        self.tsInfo.addInstall(ipo3)
+
+        po1 = FakePackage('foo')
+        po1.addRequires('bar', 'GE', (None, '3', '0'))
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('bar', version='2')
+        self.xsack.addPackage(po2)
+        po3 = FakePackage('bar', version='3')
+        self.xsack.addPackage(po3)
+        po4 = FakePackage('bar', version='4')
+        self.xsack.addPackage(po4)
+        po5 = FakePackage('bar-blah', version='2')
+        po5.addRequires('bar', 'EQ', ('0', '2', '1'))
+        self.xsack.addPackage(po5)
+        po6 = FakePackage('bar-blah', version='3')
+        po6.addRequires('bar', 'EQ', ('0', '3', '1'))
+        self.xsack.addPackage(po6)
+        po7 = FakePackage('bar-blah', version='4')
+        po7.addRequires('bar', 'EQ', ('0', '4', '1'))
+        self.xsack.addPackage(po7)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo3, po4, po7))
+
+    def test_multi_inst_dep1(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('bar-prov1', None, (None, None, None))
+        ipo1.addRequires('bar-prov2', 'EQ', ('0', '1', '0'))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addProvides('bar-prov1', None, (None, None, None))
+        po1.addProvides('bar-prov2', 'EQ', ('0', '1', '0'))
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('bar', version='2')
+        po2.addProvides('bar-prov1', None, (None, None, None))
+        po2.addProvides('bar-prov2', 'EQ', ('0', '2', '0'))
+        self.xsack.addPackage(po2)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def test_multi_inst_dep2(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('bar-prov1', None, (None, None, None))
+        ipo1.addRequires('bar-prov2', 'EQ', ('0', '1', '0'))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addProvides('bar-prov1', None, (None, None, None))
+        po1.addProvides('bar-prov2', 'EQ', ('0', '1', '0'))
+        po2 = FakePackage('bar', version='2')
+        po2.addProvides('bar-prov1', None, (None, None, None))
+        po2.addProvides('bar-prov2', 'EQ', ('0', '2', '0'))
+        self.xsack.addPackage(po2)
+        self.xsack.addPackage(po1)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def test_multi_inst_dep3(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('libbar-prov1.so.0()', None, (None, None, None))
+        ipo1.addRequires('bar-prov2', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        po1.addProvides('bar-prov2', None, (None, None, None))
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('bar', version='2')
+        po2.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        self.xsack.addPackage(po2)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def test_multi_inst_dep4(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('libbar-prov1.so.0()', None, (None, None, None))
+        ipo1.addRequires('bar-prov2', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        po1.addProvides('bar-prov2', None, (None, None, None))
+        self.xsack.addPackage(po1)
+        po2 = FakePackage('baz')
+        po2.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        self.xsack.addPackage(po2)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def test_multi_inst_dep5(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('libbar-prov1.so.0()', None, (None, None, None))
+        ipo1.addRequires('bar-prov2', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        po1.addProvides('bar-prov2', None, (None, None, None))
+        po2 = FakePackage('baz')
+        po2.addProvides('libbar-prov1.so.0()', None, (None, None, None))
+        self.xsack.addPackage(po2)
+        self.xsack.addPackage(po1)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def test_inst_require_conflict1(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('bar', None, (None, None, None))
+        ipo1.addConflicts('bar', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        self.xsack.addPackage(po1)
+
+        self.assertEquals('err', *self.resolveCode())
+
+    def test_inst_require_conflict_me1(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('bar', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        po1.addConflicts('foo', None, (None, None, None))
+        self.xsack.addPackage(po1)
+
+        self.assertEquals('err', *self.resolveCode())
+
+    def test_inst_require_obsoletes1(self):
+        ipo1 = FakePackage('foo')
+        ipo1.addRequires('bar', None, (None, None, None))
+        ipo1.addObsoletes('bar', None, (None, None, None))
+        self.tsInfo.addInstall(ipo1)
+
+        po1 = FakePackage('bar')
+        self.xsack.addPackage(po1)
+        
+        # FIXME: Does it make sense to ignore the obsoletes here? esp. as we
+        # don't ignore the conflicts above? ... I'm guessing ignoring it is
+        # by accident too? bah.
+        # self.assertEquals('err', *self.resolveCode())
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((ipo1, po1))
+
+    def testUpdate_so_req_diff_arch(self):
+        rpo1 = FakePackage('foozoomer')
+        rpo1.addRequires('libbar.so.1()', None, (None, None, None))
+        rpo1.addObsoletes('zoom', 'LT', ('8', '8', '8'))
+        self.rpmdb.addPackage(rpo1)
+        rpo2 = FakePackage('bar')
+        rpo2.addProvides('libbar.so.1()', None, (None, None, None))
+        self.rpmdb.addPackage(rpo2)
+        rpo3 = FakePackage('zoom', arch='i386')
+        self.rpmdb.addPackage(rpo3)
+
+        apo1 = FakePackage('foozoomer', version=2)
+        apo1.addRequires('libbar.so.2()', None, (None, None, None))
+        apo1.addObsoletes('zoom', 'LT', ('8', '8', '8'))
+        self.xsack.addPackage(apo1)
+        apo2 = FakePackage('bar', version=2)
+        apo2.addProvides('libbar.so.2()', None, (None, None, None))
+        self.xsack.addPackage(apo2)
+
+        self.tsInfo.addUpdate(apo2, oldpo=rpo2)
+
+        self.assertEquals('ok', *self.resolveCode())
+        self.assertResult((apo1, apo2))
+
