@@ -14,14 +14,26 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2004 Duke University
 
+"""
+Exceptions and Errors thrown by yum.
+"""
+
+from i18n import to_unicode
 
 class YumBaseError(Exception):
+    """
+    Base Yum Error. All other Errors thrown by yum should inherit from
+    this.
+    """
     def __init__(self, value=None):
         Exception.__init__(self)
         self.value = value
     def __str__(self):
         return "%s" %(self.value,)
 
+    def __unicode__(self):
+        return '%s' % to_unicode(self.value)
+
 class YumGPGCheckError(YumBaseError):
     pass
 
@@ -70,6 +82,9 @@ class UpdateError(YumBaseError):
 class RemoveError(YumBaseError):
     pass
 
+class ReinstallError(YumBaseError):
+    pass
+
 class RepoMDError(YumBaseError):
     pass
 
@@ -83,9 +98,16 @@ class MediaError(YumBaseError):
     pass
     
 class YumDeprecationWarning(DeprecationWarning):
+    """
+    Used to mark a method as deprecated.
+    """
     def __init__(self, value=None):
         DeprecationWarning.__init__(self, value)
 
 class YumFutureDeprecationWarning(YumDeprecationWarning):
+    """
+    Used to mark a method as deprecated. Unlike YumDeprecationWarning,
+    YumFutureDeprecationWarnings will not be shown on the console.
+    """
     def __init__(self, value=None):
         YumDeprecationWarning.__init__(self, value)
