@@ -24,7 +24,7 @@ import os.path
 import fnmatch
 
 import yumRepo
-from packages import PackageObject, RpmBase, YumAvailablePackage
+from packages import PackageObject, RpmBase, YumAvailablePackage, parsePackages
 import Errors
 import misc
 
@@ -34,6 +34,8 @@ import sqlutils
 import constants
 import operator
 import time
+from yum.misc import seq_max_split, to_utf8
+import sys
 
 def catchSqliteException(func):
     """This decorator converts sqlite exceptions into RepoError"""
@@ -41,7 +43,11 @@ def catchSqliteException(func):
         try:
             return func(*args, **kwargs)
         except sqlutils.sqlite.Error, e:
-            raise Errors.RepoError, str(e.message)
+            # 2.4.x requires this, but 2.6.x complains about even hasattr()
+            # of e.message ... *sigh*
+            if sys.hexversion < 0x02050000:
+                raise Errors.RepoError, str(e.message)
+            raise Errors.RepoError, str(e)
 
     newFunc.__name__ = func.__name__
     newFunc.__doc__ = func.__doc__
@@ -148,6 +154,9 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         value = r[0]
         if varname == 'epoch' and value is None:
             value = '0'
+        if varname in ('summary', 'description') and value is None:
+            # Maybe others here? ... location_base is a bad NONO though.
+            value = '' # Description for picasa, probably among others *sigh*
         if varname in {'vendor' : 1, 'packager' : 1, 'buildhost' : 1,
                        'license' : 1, 'group' : 1,
                        'summary' : 1, 'description' : 1, 'sourcerpm' : 1,
@@ -201,8 +210,10 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             # Check count(pkgId) here, the same way we do in searchFiles()?
             # Failure mode is much less of a problem.
             for ob in cur:
-                result.append( (ob['date'], _share_data(ob['author']),
-                                ob['changelog']) )
+                c_date = ob['date']
+                c_author = to_utf8(ob['author'])
+                c_log = to_utf8(ob['changelog'])
+                result.append((c_date, _share_data(c_author), c_log))
             self._changelog = result
             return
     
@@ -242,6 +253,23 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
                 self.prco[prcotype].append(_share_data(prco_set))
 
         return RpmBase.returnPrco(self, prcotype, printable)
+    
+    def _requires_with_pre(self):
+        """returns requires with pre-require bit"""
+        sql = "SELECT name, version, release, epoch, flags,pre " \
+              "FROM requires WHERE pkgKey = ?"
+        cur = self._sql_MD('primary', sql, (self.pkgKey,))
+        requires = []
+        for ob in cur:
+            pre = "0"
+            if ob['pre'].lower() in ['TRUE', 1]:
+                pre = "1"
+            prco_set = (_share_data(ob['name']), _share_data(ob['flags']),
+                        (_share_data(ob['epoch']),
+                         _share_data(ob['version']),
+                         _share_data(ob['release'])), pre)
+            requires.append(prco_set)
+        return requires
 
 class YumSqlitePackageSack(yumRepo.YumPackageSack):
     """ Implementation of a PackageSack that uses sqlite cache instead of fully
@@ -261,6 +289,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             'requires' : { },
             }
         self._key2pkg = {}
+        self._arch_allowed = None
 
     @catchSqliteException
     def _sql_MD(self, MD, repo, sql, *args):
@@ -369,8 +398,15 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         return (repo, pkgKey) in self._excludes
 
+    def _pkgArchExcluded(self, pkgarch):
+        """ Test the arch for a package against the archlist we were passed. """
+        if self._arch_allowed is not None and pkgarch not in self._arch_allowed:
+            return True
+        return False
+
     def _pkgExcluded(self, po):
-        return self._pkgKeyExcluded(po.repo, po.pkgKey)
+        return (self._pkgKeyExcluded(po.repo, po.pkgKey) or
+                self._pkgArchExcluded(po.arch))
 
     def _packageByKey(self, repo, pkgKey):
         """ Lookup a pkg by it's pkgKey, if we don't have it load it """
@@ -379,13 +415,20 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if not self._key2pkg[repo].has_key(pkgKey):
             sql = "SELECT pkgKey, pkgId, name, epoch, version, release " \
                   "FROM packages WHERE pkgKey = ?"
-            cur = self._sql_MD('primary', repo, sql, (pkgKey,))
-            po = self.pc(repo, cur.fetchone())
-            self._key2pkg[repo][pkgKey] = po
+            data = self._sql_MD('primary', repo, sql, (pkgKey,)).fetchone()
+            if data is None:
+                msg = "pkgKey %s doesn't exist in repo %s" % (pkgKey, repo)
+                raise Errors.RepoError, msg
+
+            self._key2pkg[repo][pkgKey] = self.pc(repo, data)
+        if self._pkgArchExcluded(self._key2pkg[repo][pkgKey].arch):
+            return None
         return self._key2pkg[repo][pkgKey]
         
     def _packageByKeyData(self, repo, pkgKey, data):
         """ Like _packageByKey() but we already have the data for .pc() """
+        if self._pkgArchExcluded(data['arch']):
+            return None
         if data['pkgKey'] not in self._key2pkg.get(repo, {}):
             po = self.pc(repo, data)
             self._key2pkg.setdefault(repo, {})[pkgKey] = po
@@ -431,9 +474,12 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             if self._pkgKeyExcluded(repo, ob['pkgKey']):
                 continue
             if have_data:
-                pkgs.append(self._packageByKeyData(repo, ob['pkgKey'], ob))
+                pkg = self._packageByKeyData(repo, ob['pkgKey'], ob)
             else:
-                pkgs.append(self._packageByKey(repo, ob['pkgKey']))
+                pkg = self._packageByKey(repo, ob['pkgKey'])
+            if pkg is None:
+                continue
+            pkgs.append(pkg)
         return pkgs
 
     @staticmethod
@@ -441,16 +487,43 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """ Apply SQLite escaping, if needed. Returns pattern and esc. """
         esc = ''
         if "_" in pattern or "%" in pattern:
-            esc = " ESCAPE '!'"
+            esc = ' ESCAPE "!"'
             pattern = pattern.replace("!", "!!")
             pattern = pattern.replace("%", "!%")
             pattern = pattern.replace("_", "!_")
         return (pattern, esc)
 
+    def _sql_esc_glob(self, patterns):
+        """ Converts patterns to SQL LIKE format, if required (or gives up if
+            not possible). """
+        ret = []
+        for pattern in patterns:
+            if '[' in pattern: # LIKE only has % and _, so [abc] can't be done.
+                return []      # So Load everything
+
+            # Convert to SQL LIKE format
+            (pattern, esc) = self._sql_esc(pattern)
+            pattern = pattern.replace("*", "%")
+            pattern = pattern.replace("?", "_")
+            ret.append((pattern, esc))
+        return ret
+
+    def _skip_all(self):
+        """ Are we going to skip every package in all our repos? """
+        skip_all = True
+        for repo in self.added:
+            if repo not in self._all_excludes:
+                skip_all = False
+                break
+        return skip_all
+
     @catchSqliteException
     def searchFiles(self, name, strict=False):
         """search primary if file will be in there, if not, search filelists, use globs, if possible"""
         
+        if self._skip_all():
+            return []
+
         # optimizations:
         # if it is not  glob, then see if it is in the primary.xml filelists, 
         # if so, just use those for the lookup
@@ -527,6 +600,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     @catchSqliteException
     def searchPrimaryFields(self, fields, searchstring):
         """search arbitrary fields from the primarydb for a string"""
+        if self._skip_all():
+            return []
+
         result = []
         if len(fields) < 1:
             return result
@@ -549,13 +625,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """search arbitrary fields from the primarydb for a multiple strings
            return packages, number of items it matched as a list of tuples"""
            
+        if self._skip_all():
+            return []
+
         result = [] # (pkg, num matches)
         if len(fields) < 1:
             return result
         
         # NOTE: I can't see any reason not to use this all the time, speed
         # comparison shows them as baiscally equal.
-        if len(searchstrings) > constants.PATTERNS_MAX:
+        if len(searchstrings) > (constants.PATTERNS_MAX / len(fields)):
             tot = {}
             for searchstring in searchstrings:
                 matches = self.searchPrimaryFields(fields, searchstring)
@@ -589,11 +668,17 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             for ob in cur:
                 if self._pkgKeyExcluded(rep, ob['pkgKey']):
                     continue
-                result.append((self._packageByKey(rep, ob['pkgKey']), ob['total']))
+                pkg = self._packageByKey(rep, ob['pkgKey'])
+                if pkg is None:
+                    continue
+                result.append((pkg, ob['total']))
         return result
         
     @catchSqliteException
     def returnObsoletes(self, newest=False):
+        if self._skip_all():
+            return {}
+
         if newest:
             raise NotImplementedError()
 
@@ -613,6 +698,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 # continue without processing the obsoletes
                 if self._pkgKeyExcluded(rep, ob['pkgKey']):
                     continue
+                if self._pkgArchExcluded(ob['arch']):
+                    continue
                     
                 key = ( _share_data(ob['name']), _share_data(ob['arch']),
                         _share_data(ob['epoch']), _share_data(ob['version']),
@@ -678,6 +765,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
     @catchSqliteException
     def _search(self, prcotype, name, flags, version):
+        if self._skip_all():
+            return {}
+
         if flags == 0:
             flags = None
         if type(version) in (str, type(None), unicode):
@@ -717,7 +807,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 for pkgKey, hits in tmp.iteritems():
                     if self._pkgKeyExcluded(rep, pkgKey):
                         continue
-                    result[self._packageByKey(rep, pkgKey)] = hits
+                    pkg = self._packageByKey(rep, pkgKey)
+                    if pkg is None:
+                        continue
+                    result[pkg] = hits
 
         for (rep,cache) in primarydb_items:
             if rep in self._all_excludes:
@@ -737,7 +830,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             for pkgKey, hits in tmp.iteritems():
                 if self._pkgKeyExcluded(rep, pkgKey):
                     continue
-                result[self._packageByKey(rep, pkgKey)] = hits
+                pkg = self._packageByKey(rep, pkgKey)
+                if pkg is None:
+                    continue
+                result[pkg] = hits
 
         if prcotype != 'provides' or name[0] != '/':
             if not preload:
@@ -763,7 +859,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             for ob in cur:
                 if self._pkgKeyExcluded(rep, ob['pkgKey']):
                     continue
-                result[self._packageByKey(rep, ob['pkgKey'])] = [(name, None, None)]
+                pkg = self._packageByKey(rep, ob['pkgKey'])
+                if pkg is None:
+                    continue
+                result[pkg] = [(name, None, None)]
         self._search_cache[prcotype][req] = result
         return result
 
@@ -778,6 +877,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """return a list of packages matching any of the given names. This is 
            only a match on package name, nothing else"""
         
+        if self._skip_all():
+            return []
+
+        returnList = []
+        max_entries = constants.PATTERNS_INDEXED_MAX
+        if len(names) > max_entries:
+            for names in seq_max_split(names, max_entries):
+                returnList.extend(self.searchNames(names))
+            return returnList
+
         pat_sqls = []
         qsql = """select pkgId,pkgKey,name,epoch,version,release,arch
                       from packages where """
@@ -785,7 +894,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             pat_sqls.append("name = ?")
         qsql = qsql + " OR ".join(pat_sqls)
 
-        returnList = []
         for (repo, cache) in self.primarydb.items():
             cur = cache.cursor()
             executeSQL(cur, qsql, list(names))
@@ -797,6 +905,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     @catchSqliteException
     def searchPrco(self, name, prcotype):
         """return list of packages having prcotype name (any evr and flag)"""
+        if self._skip_all():
+            return []
+
         glob = True
         querytype = 'glob'
         if not misc.re_glob(name):
@@ -944,6 +1055,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     def returnNewestByName(self, name=None, patterns=None, ignore_case=False):
         # If name is set do it from the database otherwise use our parent's
         # returnNewestByName
+        if self._skip_all():
+            return []
+
         if (not name):
             return yumRepo.YumPackageSack.returnNewestByName(self, name,
                                                              patterns,
@@ -964,6 +1078,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     # Do what packages.matchPackageNames does, but query the DB directly
     @catchSqliteException
     def matchPackageNames(self, pkgspecs):
+        if self._skip_all():
+            return [], [], []
+
         matched = []
         exactmatch = []
         unmatched = list(pkgspecs)
@@ -994,10 +1111,27 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """Builds a list of packages, only containing nevra information. No
            excludes are done at this stage. """
 
-        if patterns is None or len(patterns) > constants.PATTERNS_MAX:
+        if patterns is None:
             patterns = []
+
+        returnList = []
         
-        returnList = []        
+        fields = ['name', 'sql_nameArch', 'sql_nameVerRelArch',
+                  'sql_nameVer', 'sql_nameVerRel',
+                  'sql_envra', 'sql_nevra']
+        need_full = False
+        for pat in patterns:
+            if misc.re_full_search_needed(pat):
+                need_full = True
+                break
+
+        pat_max = constants.PATTERNS_MAX
+        if not need_full:
+            fields = ['name']
+            pat_max = constants.PATTERNS_INDEXED_MAX
+        if len(patterns) > pat_max:
+            patterns = []
+
         for (repo,cache) in self.primarydb.items():
             if (repoid == None or repoid == repo.id):
                 cur = cache.cursor()
@@ -1007,14 +1141,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
                 pat_sqls = []
                 pat_data = []
-                for pattern in patterns:
-                    for field in ['name', 'sql_nameArch', 'sql_nameVerRelArch',
-                                  'sql_nameVer', 'sql_nameVerRel',
-                                  'sql_envra', 'sql_nevra']:
+                if ignore_case:
+                    patterns = self._sql_esc_glob(patterns)
+                else:
+                    patterns = [(pat, '') for pat in patterns]
+                for (pattern, esc) in patterns:
+                    for field in fields:
                         if ignore_case:
-                            (pattern, esc) = self._sql_esc(pattern)
-                            pattern = pattern.replace("*", "%")
-                            pattern = pattern.replace("?", "_")
                             pat_sqls.append("%s LIKE ?%s" % (field, esc))
                         else:
                             pat_sqls.append("%s GLOB ?" % field)
@@ -1022,8 +1155,12 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 if pat_sqls:
                     qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
                 executeSQL(cur, qsql, pat_data)
+                #  Note: Not using _sql_pkgKey2po() so that we can "un-exclude"
+                # things later on ... if that matters.
                 for x in cur:
                     po = self._packageByKeyData(repo, x['pkgKey'], x)
+                    if po is None: # Arch exclude is done here.
+                        continue
                     returnList.append(po)
         if not patterns:
             self.pkgobjlist = returnList
@@ -1034,13 +1171,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
            packages are processed for excludes. Note that patterns is just
            a hint, we are free it ignore it. """
 
-        # Skip unused repos completely, Eg. *-source
-        skip_all = True
-        for repo in self.added:
-            if repo not in self._all_excludes:
-                skip_all = False
-
-        if skip_all:
+        if self._skip_all():
             return []
 
         if hasattr(self, 'pkgobjlist'):
@@ -1048,6 +1179,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         else:
             pkgobjlist = self._buildPkgObjList(repoid, patterns, ignore_case)
 
+        if hasattr(self, 'pkgobjlist') and patterns:
+            pkgobjlist = parsePackages(pkgobjlist, patterns, not ignore_case,
+                                       unique='repo-pkgkey')
+            pkgobjlist = pkgobjlist[0] + pkgobjlist[1]
+
         returnList = []
         for po in pkgobjlist:
             if self._pkgExcluded(po):
@@ -1059,6 +1195,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     @catchSqliteException
     def searchNevra(self, name=None, epoch=None, ver=None, rel=None, arch=None):        
         """return list of pkgobjects matching the nevra requested"""
+        if self._skip_all():
+            return []
+
         returnList = []
         
         # make sure some dumbass didn't pass us NOTHING to search on
@@ -1089,15 +1228,22 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     @catchSqliteException
     def excludeArchs(self, archlist):
         """excludes incompatible arches - archlist is a list of compat arches"""
-        
+
+        if self._arch_allowed is None:
+            self._arch_allowed = set(archlist)
+        else:
+            self._arch_allowed = self._arch_allowed.intersection(archlist)
         sarchlist = map(lambda x: "'%s'" % x , archlist)
         arch_query = ",".join(sarchlist)
 
         for (rep, cache) in self.primarydb.items():
             cur = cache.cursor()
 
-            # First of all, make sure this isn't a *-source repo or something
-            # where we'll be excluding everything.
+            #  This is a minor hack opt. for source repos. ... if they are
+            # enabled normally, we don't want to exclude each package so we
+            # check it and exclude the entire thing.
+            if not rep.id.endswith("-source") or 'src' in self._arch_allowed:
+                continue
             has_arch = False
             executeSQL(cur, "SELECT DISTINCT arch FROM packages")
             for row in cur:
@@ -1107,12 +1253,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             if not has_arch:
                 self._delAllPackages(rep)
                 return
-            
-            myq = "select pkgId, pkgKey from packages where arch not in (%s)" % arch_query
-            executeSQL(cur, myq)
-            for row in cur:
-                obj = self.pc(rep, row)
-                self.delPackage(obj)
 
 # Simple helper functions
 
